# ---------------------------------------------------------------------------
# treeline_NIR.py Created on: 2018-5-2, which is suitable to extract tree line from NIR bands mainly based on Local moran's I, coldspot analysis

# Description: extract tree line automatically from remtoe sensing images

#author: Dandan Xu, Nanjing Forestry University
# ---------------------------------------------------------------------------

# Import arcpy module
import arcpy
import arcinfo 
import os
import arcgisscripting
import types
import sys
import locale
import math

# get the input argument values
inRas = arcpy.GetParameterAsText(0)

# set output 
meadow = arcpy.GetParameterAsText(1) 
tree_line = arcpy.GetParameterAsText(2) 

#define the temporary file location
tempfiles = arcpy.GetParameterAsText(3)
if tempfiles == '#' or not tempfiles:
    tempfiles = "E:\\tree_line\\tempfiles.gdb" # provide a default value if unspecified
	
# load the arcgis extension
arcpy.CheckOutExtension("Spatial")
gp = arcgisscripting.create(10.5)
gp.OverwriteOutput = 1
gp.workspace = tempfiles
gp.scratchworkspace = tempfiles

#define workspace and scratch workspace
from arcpy.sa import *
from arcpy import env
env.overwriteOutput = True
env.workspace = tempfiles
env.scratchworkspace = tempfiles

#raster to point
inPoint = arcpy.RasterToPoint_conversion(inRas, "Value")

# Process: Cluster and Outlier Analysis (Anselin Local Morans I)
Index_Field_Name = "LMiIndex"
ZScore_Field_Name = "LMiZScore"
Probability_Field = "LMiPValue"
Cluster_Outlier_Type = "NNeighbors"
Source_ID = "OBJECTID"

inCluster = "inCluster"
arcpy.ClustersOutliers_stats(inPoint, "GRID_CODE", inCluster, "INVERSE_DISTANCE", "EUCLIDEAN_DISTANCE", "NONE", "35", "", "NO_FDR", "499")

# select the cold spot point with p value equal to 0.002
pointSel = "pointSel"
arcpy.Select_analysis(inCluster, pointSel, "\"COType\" = 'LL' AND \"LMiPValue\" = 0.002")

# calculate point density
pointDen = PointDensity(pointSel, "NONE", "30", "Rectangle 3 3 CELL", "SQUARE_METERS")

# select by point density value
InTrueRaster = 1
inFalseConstant = 0
selRas = arcpy.sa.Con((pointDen > 0.0005), InTrueRaster, inFalseConstant)

# Raster to Polygon: extract the meadow part
meadow_poly = "meadow_poly"
arcpy.RasterToPolygon_conversion(selRas, meadow_poly, "NO_SIMPLIFY", "VALUE")

# add the F_Area field
meadow_area = "meadow_area"
arcpy.CalculateAreas_stats(meadow_poly, meadow_area)

# eliminate small polygons (area smaller than 2000 square meters); must eliminate the small polygons first and then set background as null
meadow_eli = "meadow_eli"
arcpy.EliminatePolygonPart_management(meadow_area, meadow_eli, "AREA", "2000 SquareMeters", "0", "ANY")

# select only meadow area, eliminate the background 0 
meadow_sel = "meadow_sel"
arcpy.Select_analysis(meadow_eli, meadow_sel, "\"gridcode\" = 1")

# smooth the polygon
arcpy.SmoothPolygon_cartography(meadow_sel, meadow, "PAEK", 70, "", "FLAG_ERRORS")

# extract tree_line
arcpy.PolygonToLine_management(meadow, tree_line)

